/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: basicfuncs.c,v 1.2 2016/11/11 06:10:25 bigfoot Exp $
 
    Desc: Basic help functions needed by gadtools.library.
    Lang: English.
*/
#include <string.h>
#include <proto/exec.h>
#include <exec/types.h>
#include <exec/memory.h>
#include <intuition/intuition.h>
#include <intuition/screens.h>
#include <proto/graphics.h>
#include <graphics/rastport.h>
#include <graphics/rpattr.h>
#include <graphics/text.h>
#include <graphics/gfxmacros.h>
#include <proto/utility.h>
#include <utility/tagitem.h>
#include <libraries/gadtools.h>
#include <intuition/gadgetclass.h>
#include "gadtools_intern.h"

/**********************************************************************************************/

#ifdef __MORPHOS__
#else
/*
* looks buttugly..but if you want this as default:-)
*/
#define HIGH_COLOR 1  /* instead of underscore use different color to highlight key */
#endif

#define EG(x) ((struct ExtGadget *)(x))

struct GTIText
{
	struct IntuiText it;
	struct IntuiText it2;
	struct TTextAttr  ta;
	struct TTextAttr  ta2;
	UBYTE	     text[0];
};

/**********************************************************************************************/

void freeitext(struct GadToolsBase_intern *GadToolsBase,
               struct IntuiText *itext)
{
	if (itext) gtFreeVec(itext);
}

/**********************************************************************************************/

/* Create a struct IntuiText accordings to a struct NewGadget */
struct IntuiText *makeitext(struct GadToolsBase_intern *GadToolsBase,
			                            struct NewGadget *ng,
			                            struct TagItem *taglist)
{
	struct GTIText	*gtit;
	struct DrawInfo	*dri = ((struct VisualInfo *)ng->ng_VisualInfo)->vi_dri;
	struct TextFont	*font = NULL;
	struct RastPort	temprp;
	STRPTR		underscorepos;
	STRPTR		fontname;
	UWORD		fontysize;
	UBYTE		fontstyle;
	UBYTE		fontflags;
	struct TagItem      *fonttags = NULL;
	WORD		gadgettextlen;
	WORD		fontnamelen;
	WORD		underscorelen;
	WORD		alloclen;
	BOOL		fontopened = FALSE;
	UBYTE		underscore = 1; /* default for GT_Underscore = a char which hopefully a normal
						   string never contains */

	underscore = (UBYTE)GetTagData(GT_Underscore, underscore, taglist);
	underscorepos = strchr(ng->ng_GadgetText, underscore);
	gadgettextlen = strlen(ng->ng_GadgetText);

	if (ng->ng_TextAttr)
	{
		font = OpenFont(ng->ng_TextAttr);
		if (!font) return NULL;

		fontopened = TRUE;
	}
	else
	{
		font = dri->dri_Font;
	}

	fontname  = font->tf_Message.mn_Node.ln_Name;
	fontysize = font->tf_YSize;
	fontstyle = font->tf_Style;
	fontflags = font->tf_Flags;

#warning "eh, isn't this bugged? struct TextFont casted to TTextAttr.. -Zui"
#if 0
	if(fontstyle & FSF_TAGGED)
	{
		//dprintf("drawinfo has extended ttextattr bit set !!\n");

		fonttags = ((struct TTextAttr *)font)->tta_Tags;
	}
#endif

	if (!fontname) return NULL;

	fontnamelen = strlen(fontname);

	alloclen = sizeof(struct GTIText) + fontnamelen + 1 + gadgettextlen + 1 + 2; /* 2 for safety */

	gtit = (struct GTIText *)gtAllocVec(alloclen, MEMF_PUBLIC | MEMF_CLEAR);
	if (!gtit)
	{
		if (fontopened) CloseFont(font);
		return NULL;
	}

	CopyMem(fontname, gtit->text, fontnamelen);

	gtit->ta.tta_Name  = gtit->text;
	gtit->ta.tta_YSize = fontysize;
	gtit->ta.tta_Style = fontstyle;
	gtit->ta.tta_Flags = fontflags;

	if(fontstyle & FSF_TAGGED)
	{
		//dprintf("textattr is tagged, taglist %lx!!\n",fonttags);

		gtit->ta.tta_Tags = fonttags;
	}

	gtit->it.FrontPen  = dri->dri_Pens[(ng->ng_Flags & NG_HIGHLABEL) ? HIGHLIGHTTEXTPEN : TEXTPEN];
	gtit->it.BackPen   = dri->dri_Pens[BACKGROUNDPEN];
	gtit->it.DrawMode  = JAM1;
	gtit->it.LeftEdge  = 0;
	gtit->it.TopEdge   = 0;
	gtit->it.ITextFont = (struct TextAttr *) &gtit->ta;

	if (!underscorepos)
	{
		gtit->it.IText = gtit->text + fontnamelen + 1;
		gtit->it.NextText = NULL;

		if (gadgettextlen) CopyMem(ng->ng_GadgetText, gtit->it.IText, gadgettextlen);
	}
	else
	{
		gadgettextlen--;
		underscorelen = underscorepos - (STRPTR)ng->ng_GadgetText;

		gtit->it.IText = gtit->text + fontnamelen + 1;
		if (underscorelen)
		{
			CopyMem(ng->ng_GadgetText, gtit->it.IText, underscorelen);
		}
		if (gadgettextlen - underscorelen)
		{
			CopyMem(underscorepos + 1, gtit->it.IText + underscorelen, gadgettextlen - underscorelen);
		}

		gtit->it.NextText = &gtit->it2;

		gtit->it2 = gtit->it;
		gtit->it2.ITextFont = (struct TextAttr *) &gtit->ta2;
		gtit->it2.IText = gtit->it.IText + gadgettextlen + 1;
		gtit->it2.NextText = NULL;

		gtit->ta2 = gtit->ta;

#if HIGH_COLOR
		gtit->it2.FrontPen = dri->dri_Pens[(ng->ng_Flags & NG_HIGHLABEL) ? TEXTPEN : HIGHLIGHTTEXTPEN];
#else
gtit->ta2.tta_Style |= FSF_UNDERLINED;
#endif


		if (!underscorelen)
		{
			gtit->it2.LeftEdge = 0;
		}
		else
		{
			InitRastPort(&temprp);
			SetFont(&temprp, font);

			gtit->it2.LeftEdge = TextLength(&temprp, ng->ng_GadgetText, underscorelen);

			DeinitRastPort(&temprp);
		}

		gtit->it2.IText[0] = underscorepos[1];
	}

	if (fontopened) CloseFont(font);

	return &gtit->it;
}

/**********************************************************************************************/

BOOL callforlabel(struct GadToolsBase_intern *GadToolsBase,
                  struct Gadget *gad, struct RastPort *rport,
                  struct TextFont *font, LONG labelplace,
                  void (*callback)(struct callforlabeladata *))
{
	struct callforlabeladata data;
	UWORD		width, height;
	WORD		gadwidth, gadheight;

	data.GadToolsBase	=	GadToolsBase;
	data.len		=	0;
	data.gad		=	gad;
	data.rport		=	rport;
	data.x			=	0;
	data.y			=	0;

	if (EG(gad)->MoreFlags & GMORE_BOUNDS)
	{
		data.gadleft = EG(gad)->BoundsLeftEdge;
		data.gadtop  = EG(gad)->BoundsTopEdge;
		gadwidth     = EG(gad)->BoundsWidth;
		gadheight    = EG(gad)->BoundsHeight;
	}
	else
	{
		data.gadleft = gad->LeftEdge;
		data.gadtop  = gad->TopEdge;
		gadwidth     = gad->Width;
		gadheight    = gad->Height;
	}

	if (gad->GadgetText)
	{
		struct TextFont	*newfont = NULL, *oldfont;

		oldfont = rport->Font;

		/* Calculate offsets. */
		if ((gad->Flags & GFLG_LABELSTRING))
			data.text = (STRPTR)gad->GadgetText;
		else if ((gad->Flags & GFLG_LABELIMAGE))
			data.text = NULL;
		else
		{
			/* GFLG_LABELITEXT */

			struct IntuiText *itext = gad->GadgetText;

			data.text = itext->IText;

			if (itext->ITextFont)
			{
				newfont = OpenFont(itext->ITextFont);
				if (newfont)
				{
					SetFont(rport, newfont);
					SetSoftStyle(rport, itext->ITextFont->ta_Style, 0xffffffff);
				}
			}

			SetABPenDrMd(rport, itext->FrontPen, itext->BackPen, itext->DrawMode);
		}

		if (font && !newfont)
			SetFont(rport, font);
		else if (!rport->Font)
			return FALSE;

		if (data.text)
		{
			struct TextExtent te;

			data.len = strlen(data.text);
			TextExtent(rport, data.text, data.len, &te);
			width  = te.te_Width;
			height = te.te_Height;
		}
		else
		{
			width  = ((struct Image *)gad->GadgetText)->Width;
			height = ((struct Image *)gad->GadgetText)->Height;
		}

		/* 30-Jul-2003 bugfix: The offsets were adjusted to better
		 * match the original gadtools. - Piru
		 */

		if (labelplace == GV_LabelPlace_Right)
		{
			//data.x = gadwidth + 5;
			data.x = gadwidth + 7;
			data.y = (gadheight - height + 1) / 2;
		}
		else if (labelplace == GV_LabelPlace_Above)
		{
			//data.x = (gadwidth - width) / 2;
			data.x = (gadwidth - 1 - width) / 2;
			//data.y = - height - 2;
			data.y = - height - 4;
		}
		else if (labelplace == GV_LabelPlace_Below)
		{
			//data.x = (gadwidth - width) / 2;
			data.x = (gadwidth - 1 - width) / 2;
			data.y = gadheight + 3;
		}
		else if (labelplace == GV_LabelPlace_In)
		{
			//data.x = (gadwidth - width) / 2;
			data.x = (gadwidth - width + 1) / 2;
			data.y = (gadheight - height + 1) / 2;
		}
		else /* GV_LabelPlace_Left */
		{
			//data.x = - width - 4;
			data.x = - width - 8;
			data.y = (gadheight - height + 1) / 2;
		}

		callback(&data);

		if (newfont || font)
		{
			SetFont(rport, oldfont);
		}

		if (newfont)
		{
			CloseFont(newfont);
		}
	}

	return TRUE;
}

/**********************************************************************************************/

#if 1

static
void labelrendercallback(struct callforlabeladata *data)
{
	struct GadToolsBase_intern *GadToolsBase = data->GadToolsBase;
	struct Gadget *gad = data->gad;
	struct RastPort *rport = data->rport;
	LONG x = data->x;
	LONG y = data->y;
	WORD gadleft = data->gadleft;
	WORD gadtop  = data->gadtop;

	if (gad->Flags & GFLG_LABELSTRING)
	{
		SetABPenDrMd(rport, 1, 0, JAM1);
		Move(rport, gadleft + x, gadtop + y);
		Text(rport, data->text, data->len);
	}
	else if (gad->Flags & GFLG_LABELIMAGE)
	{
		DrawImage(rport, (struct Image *)gad->GadgetText, gadleft + x, gadtop + y);
	}
	else
	{

		/* 30-Jul-2003 bugfix: Now GadgetText->LeftEdge and GadgetText-TopEdge
		 * contain the text start offset, so that applications relying on it
		 * work properly (ixprefs advanced options hidden gadgets). However, for
		 * some (unknown) reasons, if I don't zero the offsets here for rendering,
		 * all other text rending goes apeshit.
		 *
		 * This way we in fact stay compatible to previous method that was ok
		 * for rending, but had bogus (0,0) offsets in GadgetText.
		 *
		 * I have no clue if this is even close to what original gadtools does.
		 * gadtools is a fucking mess. - Piru
		 */

		WORD oldleft, oldtop;

		oldleft = gad->GadgetText->LeftEdge;
		oldtop  = gad->GadgetText->TopEdge;

		gad->GadgetText->LeftEdge = 0;
		gad->GadgetText->TopEdge  = 0;

		PrintIText(rport, gad->GadgetText, gadleft + x, gadtop + y);

		gad->GadgetText->LeftEdge = oldleft;
		gad->GadgetText->TopEdge  = oldtop;
	}
}



BOOL renderlabel(struct GadToolsBase_intern *GadToolsBase,
                 struct Gadget *gad, struct RastPort *rport,
                 struct TextFont *font, LONG labelplace)
{
	return callforlabel(GadToolsBase, gad, rport, font, labelplace, labelrendercallback);
}

#else

BOOL renderlabel(struct GadToolsBase_intern *GadToolsBase,
                 struct Gadget *gad, struct RastPort *rport,
                 struct TextFont *font, LONG labelplace)
{
	struct TextFont	*newfont = NULL, *oldfont;
	struct TextExtent	te;
	STRPTR		text;
	int			len = 0, x, y;
	UWORD		width, height;
	WORD		gadleft, gadtop, gadwidth, gadheight;

	if (EG(gad)->MoreFlags & GMORE_BOUNDS)
	{
		gadleft   = EG(gad)->BoundsLeftEdge;
		gadtop    = EG(gad)->BoundsTopEdge;
		gadwidth  = EG(gad)->BoundsWidth;
		gadheight = EG(gad)->BoundsHeight;
	}
	else
	{
		gadleft   = gad->LeftEdge;
		gadtop    = gad->TopEdge;
		gadwidth  = gad->Width;
		gadheight = gad->Height;
	}

	if (gad->GadgetText)
	{
		oldfont = rport->Font;

		/* Calculate offsets. */
		if ((gad->Flags & GFLG_LABELSTRING))
			text = (STRPTR)gad->GadgetText;
		else if ((gad->Flags & GFLG_LABELIMAGE))
			text = NULL;
		else
		{
			/* GFLG_LABELITEXT */

			struct IntuiText *itext = gad->GadgetText;

			text = itext->IText;

			if (itext->ITextFont)
			{
				newfont = OpenFont(itext->ITextFont);
				if (newfont)
				{
					SetFont(rport, newfont);
					SetSoftStyle(rport, itext->ITextFont->ta_Style, 0xffffffff);
				}
			}

			SetABPenDrMd(rport, itext->FrontPen, itext->BackPen, itext->DrawMode);
		}

		if (font && !newfont)
			SetFont(rport, font);
		else if (!rport->Font)
			return FALSE;

		if (text)
		{
			len = strlen(text);
			TextExtent(rport, text, len, &te);
			width  = te.te_Width;
			height = te.te_Height;
		}
		else
		{
			width  = ((struct Image *)gad->GadgetText)->Width;
			height = ((struct Image *)gad->GadgetText)->Height;
		}

		if (labelplace == GV_LabelPlace_Right)
		{
			x = gadleft + gadwidth + 5;
			//y = gadtop + (gadheight - height) / 2 + 1;
			y = gadtop + (gadheight - height + 1) / 2;
		}
		else if (labelplace == GV_LabelPlace_Above)
		{
			x = gadleft + (gadwidth - width) / 2;
			y = gadtop - height - 2;
		}
		else if (labelplace == GV_LabelPlace_Below)
		{
			x = gadleft + (gadwidth - width) / 2;
			y = gadtop + gadheight + 3;
		}
		else if (labelplace == GV_LabelPlace_In)
		{
			x = gadleft + (gadwidth - width) / 2;
			//y = gadtop + (gadheight - height) / 2 + 1;
			y = gadtop + (gadheight - height + 1) / 2;
		}
		else /* GV_LabelPlace_Left */
		{
			x = gadleft - width - 4;
			//y = gadtop + (gadheight - height) / 2 + 1;
			y = gadtop + (gadheight - height + 1) / 2;
		}

		if (gad->Flags & GFLG_LABELSTRING)
		{
			SetABPenDrMd(rport, 1, 0, JAM1);
			Move(rport, x, y);
			Text(rport, text, len);

		}
		else if (gad->Flags & GFLG_LABELIMAGE)
		{
			DrawImage(rport, (struct Image *)gad->GadgetText, x, y);
		}
		else
		{
			PrintIText(rport, gad->GadgetText, x, y);
		}

		if (newfont || font)
			SetFont(rport, oldfont);

		if (newfont)
			CloseFont(newfont);
	}

	return TRUE;
}

#endif

/**********************************************************************************************/

void DoDisabledPattern(struct RastPort *rp, WORD x1, WORD y1, WORD x2, WORD y2,
                       WORD pen, struct GadToolsBase_intern *GadToolsBase)
{
	UWORD pattern[] = { 0x8888, 0x2222 };

	SetDrMd( rp, JAM1 );
	SetAPen( rp, pen );
	SetAfPt( rp, pattern, 1);

	/* render disable pattern */
	RectFill(rp, x1, y1, x2, y2);

	SetAfPt (rp, NULL, 0);

}

/**********************************************************************************************/
