/*
	Copyright  1995-2001, The AROS Development Team. All rights reserved.
	$Id: groupclass.c,v 1.1.1.1 2005/03/15 15:54:23 laire Exp $
 
	Desc: Internal GadTools slider class.
	Lang: English
*/

#undef AROS_ALMOST_COMPATIBLE
#define AROS_ALMOST_COMPATIBLE 1

#include <proto/exec.h>
#include <exec/libraries.h>
#include <exec/memory.h>
#include <proto/dos.h>
#include <intuition/classes.h>
#include <intuition/classusr.h>
#include <intuition/gadgetclass.h>
#include <intuition/imageclass.h>
#include <intuition/icclass.h>
#include <intuition/intuition.h>
#include <utility/tagitem.h>
#include <proto/alib.h>
#include <proto/utility.h>
#include <proto/gadtools.h>

#include <string.h> /* memset() */

#if 0
#define SDEBUG 1
#define DEBUG 1
#endif

#include "gadtools_intern.h"

/**********************************************************************************************/

#define G(x) ((struct Gadget *)(x))
#define EG(X) ((struct ExtGadget *)(x))

#define GadToolsBase ((struct GadToolsBase_intern *)cl->cl_UserData)

/**********************************************************************************************/

AROS_UFP2 (void, puttostr,
		   AROS_UFPA(UBYTE, chr, D0),
		   AROS_UFPA(STRPTR *,strPtrPtr,A3)
		  );

/**********************************************************************************************/

struct GData
{
	struct DrawInfo	*dri;
	struct TextFont	*font;
	UBYTE		labelplace;
	BOOL		was_disabled;
};

/**********************************************************************************************/

STATIC IPTR group_new(Class *cl, Object *o, struct opSet *msg)
{
	struct DrawInfo	*dri;

	D(bug("group::New:\n"));

	dri = (APTR)GetTagData(GA_DrawInfo, NULL, msg->ops_AttrList);
	if (!dri)
		return NULL;

	o = (Object *)DoSuperMethodA(cl, o, (Msg)msg);
	if (o)
	{
		struct GData *data = INST_DATA(cl, o);
		static const struct TagItem tags[] =
	    {
		    {GA_Immediate,	 TRUE},
		    {GA_RelVerify,	TRUE	},
		    {TAG_DONE			}
	    };
		struct TextAttr *tattr;

		D(bug("group::New: o %p\n", o));

		data->dri = dri;

		data->labelplace = GetTagData(GA_LabelPlace, GV_LabelPlace_Left, msg->ops_AttrList);

		data->was_disabled = FALSE;

		tattr = (APTR)GetTagData(GA_TextAttr, NULL, msg->ops_AttrList);

		if (tattr)
			data->font = OpenFont(tattr);

		SetAttrsA(o, tags);

		G(o)->GadgetType |= GTYP_GADTOOLS;

		ReturnPtr ("group::New", IPTR, (IPTR)o);
	}

	ReturnPtr ("group::New", IPTR, 0);
}

/**********************************************************************************************/

STATIC IPTR group_dispose(Class * cl, Object * o, Msg msg)
{
	struct GData *data = INST_DATA(cl, o);
	if (data->font) CloseFont(data->font);
	return DoSuperMethodA(cl, o, msg);
}

/**********************************************************************************************/

STATIC IPTR group_render(Class *cl, Object *o, struct gpRender *msg)
{
	IPTR		retval;
	struct GData	*data = INST_DATA(cl, o);
	struct gpRender	render_msg;

	render_msg.MethodID = GM_RENDER;
	render_msg.gpr_GInfo = msg->gpr_GInfo;
	render_msg.gpr_RPort = msg->gpr_RPort;
	render_msg.gpr_Redraw = msg->gpr_Redraw;

	if (data->was_disabled && !(G(o)->Flags & GFLG_DISABLED))
	{
		EraseRect(msg->gpr_RPort,
		          G(o)->LeftEdge,
		          G(o)->TopEdge,
		          G(o)->LeftEdge + G(o)->Width - 1,
		          G(o)->TopEdge + G(o)->Height - 1);
		data->was_disabled = FALSE;
		render_msg.gpr_Redraw = GREDRAW_REDRAW;
	}

	if (render_msg.gpr_Redraw == GREDRAW_REDRAW)
	{
		renderlabel(GadToolsBase, (struct Gadget *)o, msg->gpr_RPort, data->font, data->labelplace);
	}

	retval = DoSuperMethodA(cl, o, (Msg)&render_msg);

	if (G(o)->Flags & GFLG_DISABLED)
	{
		DoDisabledPattern(msg->gpr_RPort,
		                  G(o)->LeftEdge,
		                  G(o)->TopEdge,
		                  G(o)->LeftEdge + G(o)->Width - 1,
		                  G(o)->TopEdge + G(o)->Height - 1,
		                  data->dri->dri_Pens[SHADOWPEN],
		                  GadToolsBase);
		data->was_disabled = TRUE;
	}

	ReturnInt("group::Render", IPTR, retval);
}

/**********************************************************************************************/

AROS_UFH3S(IPTR, dispatch_groupclass,
		   AROS_UFHA(Class *,  cl,  A0),
		   AROS_UFHA(Object *, o,   A2),
		   AROS_UFHA(Msg,      msg, A1)
		  )
{
	AROS_USERFUNC_INIT

	IPTR retval;

	D(bug("dispatch_groupclass: cl 0x%lx o 0x%lx msg 0x%lx\n",cl,o,msg));

	switch(msg->MethodID)
	{
	case OM_NEW:
		D(bug("dispatch_groupclass: OM_NEW\n"));
		retval = group_new(cl, o, (struct opSet *)msg);
		break;

	case OM_DISPOSE:
		D(bug("dispatch_groupclass: OM_DISPOSE\n"));
		retval = group_dispose(cl, o, msg);
		break;

	case GM_RENDER:
		D(bug("dispatch_groupclass: OM_RENDER\n"));
		retval = group_render(cl, o, (struct gpRender *)msg);
		break;

	case GTM_GadgetIDCMPMsg:
		return TRUE;

	default:
		retval = DoSuperMethodA(cl, o, msg);
		break;
	} /* switch */

	D(bug("dispatch_groupclass: retval 0x%lx\n",retval));

	return (retval);

	AROS_USERFUNC_EXIT
}  /* dispatch_Gtlvclass */

/**********************************************************************************************/

#undef GadToolsBase

Class *makegroupclass(struct GadToolsBase_intern * GadToolsBase)
{
	Class *cl;

	ObtainSemaphore(&GadToolsBase->classsema);

	cl = GadToolsBase->groupclass;
	if (!cl)
	{
		cl = MakeClass(NULL, GROUPGCLASS, NULL, sizeof(struct GData), 0UL);
		if (cl)
		{
			cl->cl_Dispatcher.h_Entry = (APTR) AROS_ASMSYMNAME(dispatch_groupclass);
			cl->cl_Dispatcher.h_SubEntry = NULL;
			cl->cl_UserData = (IPTR) GadToolsBase;

			GadToolsBase->groupclass = cl;
		}
	}

	ReleaseSemaphore(&GadToolsBase->classsema);

	return cl;
}

/**********************************************************************************************/

