/*
	Copyright  1995-2001, The AROS Development Team. All rights reserved.
	$Id: gt_filterimsg.c,v 1.2 2018/07/04 20:59:35 piru Exp $
 
	Desc:
	Lang: english
*/
#include <proto/exec.h>
#include <exec/memory.h>
#include <intuition/intuition.h>
#include <intuition/imageclass.h>
#include <intuition/gadgetclass.h>

#include <proto/alib.h>
#include <string.h>

#if 0
#define SDEBUG 1
#define DEBUG 1
#endif
#include "gadtools_intern.h"


/*********************************************************************
 
	NAME */
#include <proto/gadtools.h>

AROS_LH1(struct IntuiMessage *, GT_FilterIMsg,

		 /*  SYNOPSIS */
		 AROS_LHA(struct IntuiMessage *, imsg, A1),

		 /*  LOCATION */
		 struct Library *, GadToolsBase, 17, GadTools)

/*  FUNCTION
	Processes an intuition message. Normally, you should not use this
	function and call GT_GetIMsg() instead. If this functions returns
	with a value != NULL, you have to call GT_PostFilterIMsg(), when
	you are done with processing the message. If it return a NULL
	pointer, you have to ReplyMsg() the message, you passed to
	GT_FilterIMsg().
 
	INPUTS
	imsg - pointer to the intuition message to process
 
	RESULT
	Either a pointer to a processed intuition message or NULL, in which
	case the message had only meaning to gadtools.
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
	GT_PostFilterIMsg(), GT_GetIMsg(), intuition.library/ReplyMsg()
 
	INTERNALS
 
	HISTORY
 
***************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct GadToolsBase_intern *,GadToolsBase)

	struct GT_IntuiMessage  *rc = NULL;
	struct GT_ContextGadget *contextgad;
	struct Gadget	    *gad;
	IPTR		    old_gadget_value;
	BOOL		    msg_only_for_gadtools = FALSE;
	BOOL		    found_contextgad = FALSE;

	/* Find Context Gadget. Should be first GTYP_GADTOOLS gadget
	   in windows gadgetlist */

	D(bug("FilterIMsg: imsg 0x%lx\n",imsg));

	contextgad = (struct GT_ContextGadget *)imsg->IDCMPWindow->FirstGadget;
	while (contextgad)
	{
		if (contextgad->gad.GadgetType == GTYP_GADTOOLS)
		{
			if ((contextgad->magic  == CONTEXT_MAGIC) &&
			        (contextgad->magic2 == CONTEXT_MAGIC2))
			{
				found_contextgad = TRUE;
				break;
			}
		}
		contextgad = (struct GT_ContextGadget *)contextgad->gad.NextGadget;
	}

	if (!found_contextgad) contextgad = FALSE;

	D(bug("FilterIMsg: contextgad 0x%lx\n",contextgad));

	if (contextgad)
	{
#if 0
		/* cyfm (2003/01/02): forget about that design, multiple calls to GetIMsg() without */
		/* ReplyIMsg() inbetween will break here with a static structure in the context */

		rc = &contextgad->gtmsg;

		/* copy imsg into extended gt intuimessage */

		rc->imsg = *(struct ExtIntuiMessage *)imsg;
		rc->origmsg = imsg;
		/* rc->wasalloced = FALSE; */
#else
		rc = gtAllocMem(sizeof(struct GT_IntuiMessage),MEMF_PUBLIC | MEMF_CLEAR);

		D(bug("FilterIMsg: allocated new gtmsg at 0x%lx\n",rc));

		if(rc)
		{
			/* copy imsg into extended gt intuimessage */

			/* struct copy causes memtrashing for some odd reason (cyfm) */
			/* rc->imsg = *(struct ExtIntuiMessage *)imsg; */
			memcpy(&rc->imsg,imsg,sizeof(struct ExtIntuiMessage));
			/* make sure that copied message cant be replied anymore, there are  */
			/* some stupid programs out there */
			rc->imsg.eim_IntuiMessage.ExecMessage.mn_ReplyPort = NULL;
			rc->origmsg = imsg;
			rc->wasalloced = TRUE;
		}
#endif

		if(rc)
		{
			switch(imsg->Class)
			{
			case IDCMP_GADGETDOWN:
				D(bug("FilterIMsg: IDCMP_GADGETDOWN\n"));
				contextgad->activegadget = NULL;

				gad = (struct Gadget *)imsg->IAddress;

				D(bug("FilterIMsg: GadgetType %ld (GTYP_GADTOOLS set ? %ld) (GTYP_GTYPEMASK == GTYP_CUSTOMGADGET ? %ld)\n",
				      gad->GadgetType,
				      gad->GadgetType & GTYP_GADTOOLS,
				      (gad->GadgetType & GTYP_GTYPEMASK) == GTYP_CUSTOMGADGET));
				if ((gad->GadgetType & GTYP_GADTOOLS) &&
						((gad->GadgetType & GTYP_GTYPEMASK) == GTYP_CUSTOMGADGET))
				{
					contextgad->activegadget = gad;
					contextgad->gadgetkind = 0;
					contextgad->childgadgetkind = 0;
					contextgad->scrollticker = 3;

					GetAttr(GTA_GadgetKind, (Object *)gad, &contextgad->gadgetkind);
					GetAttr(GTA_ChildGadgetKind, (Object *)gad, &contextgad->childgadgetkind);

					D(bug("FilterIMsg: gadgetkind %ld\n",contextgad->gadgetkind));
					switch(contextgad->gadgetkind)
					{
					case STRING_KIND:
						D(bug("FilterIMsg: STRING_KIND\n"));
					case SLIDER_KIND:
						D(bug("FilterIMsg: SLIDER_KIND\n"));
					case SCROLLER_KIND:
						D(bug("FilterIMsg: SCROLLER_KIND\n"));
					case LISTVIEW_KIND:
						D(bug("FilterIMsg: LISTVIEW_KIND\n"));
						msg_only_for_gadtools = DoMethod((Object *)gad,
						                                 GTM_GadgetIDCMPMsg, (ULONG) &rc->imsg.eim_IntuiMessage);
						break;

					case MX_KIND:
						D(bug("FilterIMsg: MX_KIND\n"));
						GetAttr(GTMX_Active, (Object *)gad, &contextgad->gadget_value);
						rc->imsg.eim_IntuiMessage.Code = contextgad->gadget_value;
						break;

					} /* switch(contextgad->gadgetkind) */

				} /* if gadtools gadget and customgadget */
				else
				{
					D(bug("FilterIMsg: No Gadtools custom gadget\n"));
				}
				break;

			case IDCMP_GADGETUP:
				D(bug("FilterIMsg: IDCMP_GADGETUP\n"));
				gad = (struct Gadget *)imsg->IAddress;
				if (gad == contextgad->activegadget)
				{
					switch(contextgad->gadgetkind)
					{
					case STRING_KIND:
						D(bug("FilterIMsg: STRING_KIND\n"));
					case SLIDER_KIND:
						D(bug("FilterIMsg: SLIDER_KIND\n"));
					case LISTVIEW_KIND:
						D(bug("FilterIMsg: LISTVIEW_KIND\n"));
					case SCROLLER_KIND:
						D(bug("FilterIMsg: SCROLLER_KIND\n"));
						msg_only_for_gadtools = DoMethod((Object *)gad,
						                                 GTM_GadgetIDCMPMsg, (ULONG) &rc->imsg.eim_IntuiMessage);
						break;
					} /* switch(contextgad->gadgetkind) */

				} /* if (gad == contextgad->activegadget) */

				contextgad->activegadget = NULL;
				break;

			case IDCMP_INTUITICKS:
				if (!(gad = contextgad->activegadget))
				{
					contextgad->activegadget = NULL;
				}
				else if (gad->Activation & GACT_ACTIVEGADGET)
				{
					switch (contextgad->gadgetkind)
					{
					case STRING_KIND:
					case SLIDER_KIND:
					case SCROLLER_KIND:
					case LISTVIEW_KIND:
						rc->imsg.eim_IntuiMessage.IAddress = gad;
						msg_only_for_gadtools = DoMethod((Object *)gad,
						                                 GTM_GadgetIDCMPMsg, (ULONG) &rc->imsg.eim_IntuiMessage);
						break;

					} /* switch (contextgad->gadgetkind) */
				} /* else if (gad->Activation & GACT_ACTIVEGADGET) */
				break;

			case IDCMP_MOUSEMOVE:
				if (!(gad = contextgad->activegadget))
				{
					contextgad->activegadget = NULL;
				}
				else if (gad->Activation & GACT_ACTIVEGADGET)
				{
					/* gadget is still active */

					old_gadget_value = contextgad->gadget_value;

					switch (contextgad->gadgetkind)
					{
					case STRING_KIND:
					case SLIDER_KIND:
					case SCROLLER_KIND:
					case LISTVIEW_KIND:
						rc->imsg.eim_IntuiMessage.IAddress = gad;
						msg_only_for_gadtools = DoMethod((Object *)gad,
						                                 GTM_GadgetIDCMPMsg, (ULONG) &rc->imsg.eim_IntuiMessage);
						break;

					} /* switch (contextgad->gadgetkind) */

				} /* gadget is active */
				break;

			case IDCMP_VANILLAKEY:
			case IDCMP_RAWKEY:
				if ((rc->imsg.eim_IntuiMessage.Code >= NM_WHEEL_UP) && (rc->imsg.eim_IntuiMessage.Code <= NM_WHEEL_DOWN))
				{
					struct Gadget *gadtoolsgad = NULL;
					struct Gadget *listgad = NULL;
					BOOL manylists = FALSE;
					BOOL mouseover = FALSE;
					ULONG kind;

					for (gadtoolsgad = rc->imsg.eim_IntuiMessage.IDCMPWindow->FirstGadget; gadtoolsgad; gadtoolsgad = gadtoolsgad->NextGadget)
					{
						/* is this a gadtools gadget? */
						if (!(gadtoolsgad->GadgetType & GTYP_GADTOOLS)) continue;
						if (!(gadtoolsgad->GadgetType & GTYP_CUSTOMGADGET)) continue;
						if (!(gadtoolsgad->MutualExclude)) continue;

						/* NOTE: quite tricky, but should be OK */
						GetAttr(GTA_GadgetKind, (Object *)gadtoolsgad, &kind);

						/* if LISTVIEW_KIND check if list is below mouse, if there's only one list
						then we do not care about mouse position! */

#define CALCX(win,gad) ((gad->LeftEdge < 0) ? (win->Width + gad->LeftEdge) : gad->LeftEdge)
#define CALCY(win,gad) ((gad->TopEdge < 0) ? (win->Height + gad->TopEdge) : gad->TopEdge)
#define CALCW(win,gad) ((gad->Width < 0) ? (win->Width + gad->Width) : gad->Width)
#define CALCH(win,gad) ((gad->Height < 0) ? (win->Height + gad->Height) : gad->Height)
#define ISOVERMOUSEGADGET(win,gad) ((((win->MouseX >= CALCX(win,gad)) && (win->MouseX < CALCX(win,gad) + CALCW(win,gad))) && \
						((win->MouseY >= CALCY(win,gad)) && (win->MouseY < CALCY(win,gad) + CALCH(win,gad)))) ? TRUE : FALSE)
						
						if ((kind == LISTVIEW_KIND) && OCLASS((Object *)gadtoolsgad) == ((struct GadToolsBase_intern *)(GadToolsBase))->listviewgclass)
						{
							if (listgad) manylists = TRUE;

							if (listgad)
							{
								if (ISOVERMOUSEGADGET(rc->imsg.eim_IntuiMessage.IDCMPWindow,gadtoolsgad))
								{
									listgad = gadtoolsgad;
									mouseover = TRUE;
								}
							} else {
								if (ISOVERMOUSEGADGET(rc->imsg.eim_IntuiMessage.IDCMPWindow,gadtoolsgad))
								{
									mouseover = TRUE;
								}
								listgad = gadtoolsgad;
							}
						}

					}

					/* if there are multiple lists we only do scrolling when mouse is over one,
					otherwise we scroll in the only list no matter what position mouse ptr has */
					if (listgad && ((manylists && mouseover) || !manylists))
					{
						rc->imsg.eim_IntuiMessage.IAddress = listgad;
						DoMethod((Object *)listgad,GTM_GadgetIDCMPMsg, (ULONG) &rc->imsg.eim_IntuiMessage);
						msg_only_for_gadtools = TRUE;
					}
				}
				break;

			default:
				break;
			} /* switch (imsg->Class) */
		}

		if (msg_only_for_gadtools)
		{
			/* its not really necessary to call GT_PFIMsg here
			   , in the actual implementation */

			D(bug("FilterIMsg: Gadtools private msg\n"));
			GT_PostFilterIMsg((struct IntuiMessage *)rc);

			/* this tells GT_GetImsg that imsg was for GadTools
			   use only = it calls ReplyMsg and does not hand
			   over the msg to the app */

			rc = NULL;
		}

	} /* if (contextgad) */
	else
	{
		/* no context gadget = stupid coder which uses
		   GT_GetIMsg without having gadtools gadgets
		   in the window. We must alloc a GT_IntuiMessage
		   because GT_PostFilterIMsg (which is called
		   by GT_ReplyMsg) relies on this extended structure */

		D(bug("FilterIMsg: no context gadget\n"));
		rc = gtAllocMem(sizeof(struct GT_IntuiMessage),MEMF_PUBLIC | MEMF_CLEAR);
		if (rc)
		{
			/* copy imsg into extended gt intuimessage */

			/* struct copy causes memtrashing for some odd reason (cyfm) */
			/* rc->imsg = *(struct ExtIntuiMessage *)imsg; */
			memcpy(&rc->imsg,imsg,sizeof(struct ExtIntuiMessage));

			/* make sure that copied message cant be replied anymore, there are  */
			/* some stupid programs out there */
			rc->imsg.eim_IntuiMessage.ExecMessage.mn_ReplyPort = NULL;

			rc->origmsg = imsg;
			rc->wasalloced = TRUE;
		}

		/* if allocation fails, GT_GetIMsg thinks msg was for
		   GadTools use only and replies the origmsg. So no
		   bad things will happen. The only problem is that
		   the app will not get the msg. */

	} /* if (contextgad) else */

	return (struct IntuiMessage *)rc;

	AROS_LIBFUNC_EXIT

} /* GT_FilterIMsg */
