/*
    (C) 2002 MorphOS The MorphOS Development Team, All Rights Reserved.
    $Id: mempool.c,v 1.4 2007/08/18 21:58:40 piru Exp $
 
    Desc:
    Lang: english
*/

#define	DEBUG_MEMPOOL(x)		;

#undef SysBase
#include "mempool.h"

#include <exec/memory.h>
#include <exec/execbase.h>
#include <exec/alerts.h>
#include <dos/dos.h>
#include <dos/dosextens.h>

#include <proto/exec.h>
#include <hardware/atomic.h>

#ifdef __MORPHOS__

#include <public/proto/libmemblock/memblock_protos.h>

void	dprintf(char *, ...) __attribute__ ((format (printf, 1, 2)));

#endif /* __MORPHOS__ */


struct __poolmeminter
{
	ULONG   pmi_AllocSize;
};
#define	PMIF_SEMAMEM            (1 << 31)
#define PMIF_SIZEMASK           ((ULONG) ~PMIF_SEMAMEM)
#define	PMIF_MAXSIZE            ((ULONG) (PMIF_SEMAMEM - sizeof(struct __poolmeminter) - 1))

struct __poolmempool
{
	struct ExecBase        *pmp_SysBase;
	APTR                    pmp_SemaMemPool;
	APTR                    pmp_ForbidMemPool;
	APTR                    pmp_SemaMemPoolDelList;
	ULONG			pmp_Garbagecollect;
	struct SignalSemaphore  pmp_MemPoolSema;
	UWORD                   pmp_Attributes;
};

#define POOLPTR(x) struct __poolmempool *Pool = (struct __poolmempool *) (x)


#ifdef __MORPHOS__

#define __poolClearMem memclr

#else /* __MORPHOS__ */

static
void __poolClearMem(APTR ptr, ULONG size)
{
	ULONG quads;

	//DEBUG_MEMPOOL(dprintf("__poolClearMem: clear %lu bytes at 0x%lx\n", size, (ULONG) ptr));

	if (size >= 64)
	{
		ULONG *longs;

		longs = (ULONG *) ptr;

		quads = size >> 4;
		size &= (ULONG) (sizeof(ULONG) * 4 - 1);

		do
		{
			*longs++ = 0;
			*longs++ = 0;
			*longs++ = 0;
			*longs++ = 0;
		}
		while (--quads);

		quads = size >> 2;
		size &= (ULONG) (sizeof(ULONG) - 1);

		while (quads--)
		{
			*longs++ = 0;
		}

		ptr = longs;
	}

	if (size)
	{
		UBYTE *bytes;

		bytes = (UBYTE *) ptr;

		do
		{
			*bytes++ = 0;
		}
		while (--size);
	}

	//DEBUG_MEMPOOL(dprintf("__poolClearMem: return\n"));
}


#endif /* __MORPHOS__ */


/* NOTE: must be called single threaded
 */
BOOL poolCreate(struct ExecBase *SysBase, APTR *PoolPtr,
                ULONG attributes, ULONG puddleSize, ULONG threshSize)
{
	BOOL res = FALSE;

	DEBUG_MEMPOOL(dprintf("poolCreate: SysBase 0x%lx PoolPtr 0x%lx attributes 0x%lx puddleSize %lu threshSize %lu\n",
	                      (ULONG) SysBase, (ULONG) PoolPtr, attributes, puddleSize, threshSize));

	if (PoolPtr && (threshSize <= puddleSize))
	{
		if (*PoolPtr)
		{
			res = TRUE;
		}
		else
		{
			struct __poolmempool *Pool;

			if (SysBase->LibNode.lib_Version >= 39L &&
			        ((Pool = AllocMem(sizeof(struct __poolmempool), MEMF_CLEAR | MEMF_PUBLIC))))
			{
				Pool->pmp_SysBase = SysBase;

				attributes &= (ULONG) ~MEMF_CLEAR;

				Pool->pmp_SemaMemPool = CreatePool(attributes, puddleSize, threshSize);
				if (Pool->pmp_SemaMemPool)
				{
					Pool->pmp_ForbidMemPool = CreatePool(attributes, puddleSize, threshSize);
					if (Pool->pmp_ForbidMemPool)
					{
						__poolClearMem(&Pool->pmp_MemPoolSema, sizeof(struct SignalSemaphore));
						InitSemaphore(&Pool->pmp_MemPoolSema);

						//Pool->pmp_Garbagecollect = 0;
						Pool->pmp_Attributes = attributes & (UWORD) ~MEMF_PUBLIC;

						DEBUG_MEMPOOL(dprintf("poolCreate: pmp_Attributes 0x%lx\n", (ULONG) Pool->pmp_Attributes));

						*PoolPtr = Pool;
						res = TRUE;
					}
				}

				if (!res)
				{
					poolDelete((APTR *) &Pool);
				}
			}
		}
	}

	DEBUG_MEMPOOL(dprintf("poolCreate: return %ld Pool 0x%lx\n", (LONG) res, res ? (ULONG) *PoolPtr : 0));
	return res;
}

#define SysBase Pool->pmp_SysBase


/* NOTE: must be called single threaded
 *       it's callers responsibility to make sure no-one
 *       is using the memory at this point.
 */
void poolDelete(APTR *PoolPtr)
{
	DEBUG_MEMPOOL(dprintf("poolDelete: PoolPtr 0x%lx Pool 0x%lx\n", (ULONG) PoolPtr, PoolPtr ? (ULONG) *PoolPtr : 0));
	if (PoolPtr && *PoolPtr)
	{
		struct __poolmempool *Pool = *((struct __poolmempool **) PoolPtr);
		*PoolPtr = NULL;

		if (Pool->pmp_ForbidMemPool)
		{
			DeletePool(Pool->pmp_ForbidMemPool);
		}
		if (Pool->pmp_SemaMemPool)
		{
			DeletePool(Pool->pmp_SemaMemPool);
		}
		FreeMem(Pool, sizeof(struct __poolmempool));

		/* SysBase pointer is no longer valid
		 */
	}

	DEBUG_MEMPOOL(dprintf("poolDelete: done\n"));
}


/* Free pending memory chunks. Must be called semaphore locked!
 */
static
void __pooldelayedfree(struct __poolmempool *Pool)
{
	struct __poolmeminter *pmi;
	APTR *next, *listptr;

	DEBUG_MEMPOOL(dprintf("__pooldelayedfree: entry\n"));

	/* This counter bump & check is not single threaded.
	   As this is just a simple counter to avoid extra
	   load, it doesn't matter...
	 */
	Pool->pmp_Garbagecollect++;
	if (Pool->pmp_Garbagecollect & 0x7)
	{
		DEBUG_MEMPOOL(dprintf("__pooldelayedfree: do no work yet, return\n"));
		return;
	}

	DEBUG_MEMPOOL(dprintf("__pooldelayedfree: doing garbage collection now\n"));

	listptr = &Pool->pmp_SemaMemPoolDelList;

	/* Figure out of there are any delayed poolFreeMem calls, and do them now if there are
	 */
	/* Note: Atomic fetch/store nicely avoids need for Forbid() */
	next = (APTR) ATOMIC_STORE((LONG *) listptr, NULL);
	if (next)
	{
		do
		{
			listptr = next;
			next = *listptr;

			pmi = ((struct __poolmeminter *) listptr) - 1;
			DEBUG_MEMPOOL(dprintf("__pooldelayedfree: FreePooled 0x%08lx %lu\n", (ULONG) pmi, pmi->pmi_AllocSize & PMIF_SIZEMASK));
			FreePooled(Pool->pmp_SemaMemPool, pmi, pmi->pmi_AllocSize & PMIF_SIZEMASK);

		}
		while (next);
	}

	DEBUG_MEMPOOL(dprintf("__pooldelayedfree: return\n"));
}


static
APTR __poolgenalloc(struct __poolmempool *Pool, ULONG byteSize, ULONG attributes)
{
	APTR ret = NULL;

	if (byteSize && byteSize <= PMIF_MAXSIZE &&
	        ((attributes & (UWORD) ~MEMF_PUBLIC) == Pool->pmp_Attributes))
	{
		struct Process *me = (struct Process *) FindTask(NULL);
		BOOL SemaOk;

		byteSize += sizeof(struct __poolmeminter);

		Forbid();
		SemaOk = (SysBase->TDNestCnt < 1 &&
		          SysBase->IDNestCnt < 0) ||
		         Pool->pmp_MemPoolSema.ss_QueueCount == -1 ||
		         Pool->pmp_MemPoolSema.ss_Owner == &me->pr_Task;

		if (SemaOk)
		{
			Permit();

			DEBUG_MEMPOOL(dprintf("__poolgenalloc: SemaOk==TRUE, Allocating memory from SemaMemPool.\n"));

			//DEBUG_MEMPOOL(dprintf("__poolgenalloc: ObtainSemaphore 0x%lx\n", (ULONG) &Pool->pmp_MemPoolSema));
			ObtainSemaphore(&Pool->pmp_MemPoolSema);


			/* Handle possible delayed __poolgenfree calls
			 */
			__pooldelayedfree(Pool);


			//DEBUG_MEMPOOL(dprintf("__poolgenalloc: AllocPooled MemPoolPtr 0x%lx\n", (ULONG) Pool->pmp_MemPoolPtr));
			ret = AllocPooled(Pool->pmp_SemaMemPool, byteSize);

			//DEBUG_MEMPOOL(dprintf("__poolgenalloc: ReleaseSemaphore 0x%lx\n", (ULONG) &Pool->pmp_MemPoolSema));
			ReleaseSemaphore(&Pool->pmp_MemPoolSema);
		}
		else
		{
			DEBUG_MEMPOOL(dprintf("__poolgenalloc: SemaOk==FALSE, Allocating memory from ForbidMemPool.\n"));

			ret = AllocPooled(Pool->pmp_ForbidMemPool, byteSize);

			Permit();
		}

		if (ret)
		{
			struct __poolmeminter *pmi;

			pmi = (struct __poolmeminter *) ret;
			ret = pmi + 1;

			pmi->pmi_AllocSize = byteSize | (SemaOk ? PMIF_SEMAMEM : 0);

			if (attributes & MEMF_CLEAR)
			{
				__poolClearMem(ret, byteSize - sizeof(struct __poolmeminter));
			}
		}
		else
		{
			if (me->pr_Task.tc_Node.ln_Type == NT_PROCESS)
			{
				me->pr_Result2 = ERROR_NO_FREE_STORE;
			}
		}
	}

	return ret;
}


static
void __poolgenfree(struct __poolmempool *Pool, struct __poolmeminter *pmi)
{
	APTR memoryBlock;
	ULONG byteSize;

	memoryBlock = pmi;
	byteSize = pmi->pmi_AllocSize & PMIF_SIZEMASK;

	if (pmi->pmi_AllocSize & PMIF_SEMAMEM)
	{
		BOOL SemaOk;

		DEBUG_MEMPOOL(dprintf("__poolgenfree: PMIF_SEMAMEM set, releasing memory from SemaMemPool.\n"));

		Forbid();
		SemaOk = (SysBase->TDNestCnt < 1 &&
		          SysBase->IDNestCnt < 0) ||
		         Pool->pmp_MemPoolSema.ss_QueueCount == -1 ||
		         Pool->pmp_MemPoolSema.ss_Owner == FindTask(NULL);

		/* If (Forbid() or Disable()) and semaphore not free and not owned by us
		 */
		if (!SemaOk)
		{
			DEBUG_MEMPOOL(dprintf("__poolgenfree: Memory allocated outside Forbid() is released within Forbid(). Added the memory to delayed freemem list.\n"));

			Permit();

			ATOMIC_SADDHEAD(&Pool->pmp_SemaMemPoolDelList, pmi + 1);

			/* All done. As you can see, we aren't really deallocating
			 * anything in this case.
			 */
			return;
		}

		Permit();

		DEBUG_MEMPOOL(dprintf("__poolgenfree: ObtainSemaphore 0x%lx\n", (ULONG) &Pool->pmp_MemPoolSema));
		ObtainSemaphore(&Pool->pmp_MemPoolSema);


		/* Handle possible delayed __poolgenfree calls
		 */
		__pooldelayedfree(Pool);


		DEBUG_MEMPOOL(dprintf("__poolgenfree: FreePooled 0x%08lx %lu\n", (ULONG) memoryBlock, byteSize));
		FreePooled(Pool->pmp_SemaMemPool, memoryBlock, byteSize);

		DEBUG_MEMPOOL(dprintf("__poolgenfree: ReleaseSemaphore 0x%lx\n", (ULONG) &Pool->pmp_MemPoolSema));
		ReleaseSemaphore(&Pool->pmp_MemPoolSema);
	}
	else
	{
		DEBUG_MEMPOOL(dprintf("__poolgenfree: PMIF_SEMAMEM clear, releasing memory from ForbidMemPool.\n"));

		DEBUG_MEMPOOL(dprintf("__poolgenfree: FreePooled 0x%08lx %lu\n", (ULONG) memoryBlock, byteSize));
		Forbid();
		FreePooled(Pool->pmp_ForbidMemPool, memoryBlock, byteSize);
		Permit();
	}
}


APTR poolAllocMem(APTR __Pool, ULONG byteSize, ULONG attributes)
{
	APTR ret = NULL;

	DEBUG_MEMPOOL(dprintf("poolAllocMem: Pool 0x%lx byteSize %lu, attributes 0x%lx\n", (ULONG) __Pool, byteSize, attributes));
	//DEBUG_MEMPOOL(dprintf("SysBase 0x%lx IDNestCnt: %ld TDNestCnt: %ld\n", (ULONG) SysBase, (LONG) SysBase->IDNestCnt, (LONG) SysBase->TDNestCnt));

	if (__Pool && byteSize)
	{
		POOLPTR(__Pool);

		ret = __poolgenalloc(Pool, byteSize, attributes);
	}

	DEBUG_MEMPOOL(dprintf("poolAllocMem: return 0x%lx\n", (ULONG) ret));

	return ret;
}


void poolFreeMem(APTR __Pool, APTR memoryBlock, ULONG byteSize)
{
	DEBUG_MEMPOOL(dprintf("poolFreeMem: Pool 0x%lx memoryBlock 0x%lx byteSize %lu\n", (ULONG) __Pool, (ULONG) memoryBlock, byteSize));
	//DEBUG_MEMPOOL(dprintf("SysBase 0x%lx IDNestCnt: %ld TDNestCnt: %ld\n", (ULONG) SysBase, (LONG) SysBase->IDNestCnt, (LONG) SysBase->TDNestCnt));

	if (__Pool && memoryBlock)
	{
		POOLPTR(__Pool);
		struct __poolmeminter *pmi;

		pmi = ((struct __poolmeminter *) memoryBlock) - 1;
		memoryBlock = pmi;
		byteSize += sizeof(struct __poolmeminter);

		/* Free bonus: Extra validity check
		 */
		if (byteSize != (pmi->pmi_AllocSize & PMIF_SIZEMASK))
		{
			DEBUG_MEMPOOL(dprintf("poolFreeMem: ***BUG*** byteSize (%lu) doesn't match with AllocSize (%lu)! Memory not released!\n", byteSize, pmi->pmi_AllocSize & PMIF_SIZEMASK));

			/* FIXME: Should allocate this from exec/alerts.h
			 */
			Alert(AT_Recovery | AN_GadTools | 0x00000020);

			return;
		}

		__poolgenfree(Pool, pmi);
	}

	DEBUG_MEMPOOL(dprintf("poolFreeMem: return\n"));
}


APTR poolAllocVec(APTR __Pool, ULONG byteSize, ULONG attributes)
{
	APTR ret = NULL;

	DEBUG_MEMPOOL(dprintf("poolAllocVec: Pool 0x%lx byteSize %lu, attributes 0x%lx\n", (ULONG) __Pool, byteSize, attributes));
	//DEBUG_MEMPOOL(dprintf("SysBase 0x%lx IDNestCnt: %ld TDNestCnt: %ld\n", (ULONG) SysBase, (LONG) SysBase->IDNestCnt, (LONG) SysBase->TDNestCnt));

	if (__Pool && byteSize)
	{
		POOLPTR(__Pool);

		ret = __poolgenalloc(Pool, byteSize, attributes);
	}

	DEBUG_MEMPOOL(dprintf("poolAllocVec: return 0x%lx\n", (ULONG) ret));

	return ret;
}


void poolFreeVec(APTR __Pool, APTR memoryBlock)
{
	DEBUG_MEMPOOL(dprintf("poolFreeVec: memoryBlock 0x%lx\n", (ULONG) memoryBlock));
	//DEBUG_MEMPOOL(dprintf("SysBase 0x%lx IDNestCnt: %ld TDNestCnt: %ld\n", (ULONG) SysBase, (LONG) SysBase->IDNestCnt, (LONG) SysBase->TDNestCnt));

	if (__Pool && memoryBlock)
	{
		POOLPTR(__Pool);
		__poolgenfree(Pool, ((struct __poolmeminter *) memoryBlock) - 1);
	}

	DEBUG_MEMPOOL(dprintf("poolFreeVec: return\n"));
}
