/*
	Copyright  1995-2001, The AROS Development Team. All rights reserved.
	$Id: scrollergclass.c,v 1.1.1.1 2005/03/15 15:54:23 laire Exp $
 
	Desc: Internal GadTools scroller class.
	Lang: English
*/

#undef AROS_ALMOST_COMPATIBLE
#define AROS_ALMOST_COMPATIBLE 1

#include <proto/exec.h>
#include <exec/libraries.h>
#include <exec/memory.h>
#include <proto/dos.h>
#include <intuition/classes.h>
#include <intuition/classusr.h>
#include <intuition/gadgetclass.h>
#include <intuition/imageclass.h>
#include <intuition/icclass.h>
#include <intuition/intuition.h>
#include <utility/tagitem.h>
#include <proto/alib.h>
#include <proto/utility.h>
#include <proto/gadtools.h>

#include <string.h> /* memset() */

#if 0
#define SDEBUG 1
#define DEBUG 1
#endif

#include "gadtools_intern.h"

/**********************************************************************************************/

#define G(x) ((struct Gadget *)(x))
#define EG(x) ((struct ExtGadget *)(x))

#define GadToolsBase ((struct GadToolsBase_intern *)cl->cl_UserData)

/**********************************************************************************************/

struct SGData
{
	Object		*scroller;
	//Object		*arrow_inc;
	//Object		*arrow_dec;
	UWORD		code;
	BOOL		immediate;
	BOOL		relverify;
};

/**********************************************************************************************/

STATIC IPTR scrollerg_new(Class *cl, Object *o, struct opSet *msg)
{
	Object *scroller = NULL,
	                   *arrow_dec = NULL,
	                                *arrow_inc = NULL ;
	struct TagItem *tag, *taglist;
	BOOL immediate = FALSE, relverify = FALSE;

	/* Tags for the scrollerg */

	struct TagItem sgtags[] =
	    {
		    {GA_FollowMouse	 , TRUE},
		    {TAG_MORE	, (IPTR)NULL	}
	    };

	/* Tags for the scroller */

	struct TagItem stags[] =
	    {
		    {GA_Left , BORDERPROPSPACINGX},
		    {GA_Top		, BORDERPROPSPACINGY},
		    {GA_Width	, 0		},
		    {GA_Height	, 0		},
		    {GA_DrawInfo	, (IPTR)NULL	},
		    {GTSC_Top	, 0		},
		    {GTSC_Total	, 0		},
		    {GTSC_Visible	, 2		},
		    {PGA_Freedom	, FREEHORIZ	},
		    {PGA_Borderless	, TRUE		},
#ifdef __MORPHOS__
		    {PGA_NewLook    , FALSE         },
#else
			{PGA_NewLook    , TRUE          },
#endif
		    {PGA_NotifyBehaviour, PG_BEHAVIOUR_NICE },
		    {PGA_RenderBehaviour, PG_BEHAVIOUR_NICE },
		    {TAG_IGNORE	,		}, /* ICA_TARGET */
		    {TAG_IGNORE	,		}, /* ICA_MAP */
		    {TAG_DONE			}
	    };

	/* Interconnection maps */

	static const struct TagItem inc_map[] =
	    {
		    {GA_ID		 , GTA_Scroller_Inc},
		    {TAG_DONE			}
	    };

	static const struct TagItem dec_map[] =
	    {
		    {GA_ID		 , GTA_Scroller_Dec},
		    {TAG_DONE			}
	    };

	/* Tags for the arrows */

	struct TagItem atags[] =
	    {
		    {GA_Left , 0},
		    {GA_Top		, 0		},
		    {GA_Width	, 0		},
		    {GA_Height	, 0		},
		    {GA_DrawInfo	, (IPTR)NULL	},
		    {GA_RelVerify	, FALSE		},
		    {GA_Immediate	, FALSE		},
		    {GA_ID		, 1		},
		    {ICA_TARGET	, (IPTR)NULL	},
		    {ICA_MAP	, (IPTR)NULL	},
		    {GTA_Arrow_Type , 0		},
		    {TAG_DONE			}
	    };


	struct opSet newmsg;
	int width, height;
	UWORD freedom;
	WORD arrowdim = 0;

	EnterFunc(bug("scrollerg::New\n"));

	/* Prevent gcc warning, width/height tags *are* there. */
	width = 0;
	height = 0;

	/* Could use GetTagData(), but this is faster */
	taglist = msg->ops_AttrList;
	while ((tag = NextTagItem(&taglist)))
	{
		IPTR tidata = tag->ti_Data;

		switch (tag->ti_Tag)
		{
		case GA_Width:
			width = (int)tidata;
			break;
		case GA_Height:
			height = (int)tidata;
			break;

		case GA_RelVerify:
			relverify = tidata != 0;
			break;
		case GA_Immediate:
			immediate = tidata != 0;
			break;

		case GA_DrawInfo:
			stags[4].ti_Data = tidata;
			atags[4].ti_Data = tidata;
			break;

		case GTSC_Top:
			stags[5].ti_Data = tidata;
			break;
		case GTSC_Total:
			stags[6].ti_Data = tidata;
			break;
		case GTSC_Visible:
			stags[7].ti_Data = tidata;
			break;
		case PGA_Freedom:
			if (tidata == LORIENT_HORIZ)
				stags[8].ti_Data = FREEHORIZ;
			else
				stags[8].ti_Data = FREEVERT;
			break;

		case GTSC_Arrows:
			arrowdim = (WORD)tidata;
			break;

		case ICA_TARGET:
			stags[13] = *tag;
			break;
		case ICA_MAP:
			stags[14] = *tag;
			break;
		}

	} /* while (iterate taglist) */

	freedom = stags[8].ti_Data;

	sgtags[1].ti_Data = (IPTR)msg->ops_AttrList;

	/* Substract the arrow's total size from the sroller's size */

	if (arrowdim == -1)
	{
		DEBUG_CREATESCROLLER(bug("scrollerg::New: default arrowdim\n"));
		if (freedom == FREEVERT)
		{
			arrowdim = width;
			DEBUG_CREATESCROLLER(bug("scrollerg::New: freevert arrowdim %ld\n", arrowdim));
		}
		else
		{
			arrowdim = height;
			DEBUG_CREATESCROLLER(bug("scrollerg::New: freehoriz arrowdim %ld\n", arrowdim));
		}
		DEBUG_CREATESCROLLER(bug("scrollerg::New: arrowdim %ld\n", arrowdim));
	}

	if (freedom == FREEVERT)
		height -= 2 * arrowdim;
	else
		width -= 2 * arrowdim;

	stags[2].ti_Data = width - BORDERPROPSPACINGX * 2; /* GA_Width */
	stags[3].ti_Data = height- BORDERPROPSPACINGY * 2; /* GA_Height */

	scroller = NewObjectA(GadToolsBase->scrollerclass, NULL, stags);

	DEBUG_CREATESCROLLER(bug("scrollerg::New: scroller %p\n", scroller));

	if (!scroller)
		return (IPTR) NULL;


	if (arrowdim) /* Scroller has arrows ? */
	{
		atags[8].ti_Data = (IPTR)scroller; /* ICA_TARGET */
		atags[9].ti_Data = (IPTR)&dec_map; /* ICA_MAP */

		if (freedom == FREEVERT)
		{
			DEBUG_CREATESCROLLER(bug("scrollerg::New: Freedom=FREEVERT\n"));

			atags[0].ti_Data = 0;	/* GA_Left */
			atags[1].ti_Data = height;	/* GA_Top */
			atags[2].ti_Data = width;	/* GA_Width */
			atags[3].ti_Data = arrowdim;/* GA_Height */
			atags[10].ti_Data = UPIMAGE;

			arrow_dec = NewObjectA(GadToolsBase->arrowclass, NULL, atags);
			if (!arrow_dec)
				goto failure;

			DEBUG_CREATESCROLLER(bug("scrollerg::New: arrow_dec gadget 0x%lx\n",arrow_dec));

			atags[1].ti_Data += arrowdim;/* GA_Top */
			atags[9].ti_Data = (IPTR)&inc_map; /* ICA_MAP */
			atags[10].ti_Data = DOWNIMAGE;

			arrow_inc = NewObjectA(GadToolsBase->arrowclass, NULL, atags);
			if (!arrow_inc)
				goto failure;

			DEBUG_CREATESCROLLER(bug("scrollerg::New: arrow_inc gadget 0x%lx\n",arrow_inc));
		}
		else
		{
			DEBUG_CREATESCROLLER(bug("scrollerg::New: Freedom=FREEHORIZ\n"));

			atags[0].ti_Data = width;	/* GA_Left */
			atags[1].ti_Data = 0;	/* GA_Top */
			atags[2].ti_Data = arrowdim;/* GA_Width */
			atags[3].ti_Data = height;	/* GA_Height */
			atags[10].ti_Data = LEFTIMAGE;

			arrow_dec = NewObjectA(GadToolsBase->arrowclass, NULL, atags);
			if (!arrow_dec)
				goto failure;

			DEBUG_CREATESCROLLER(bug("scrollerg::New: arrow_dec gadget 0x%lx\n",arrow_dec));

			atags[0].ti_Data += arrowdim;/* GA_Left */
			atags[9].ti_Data = (IPTR)&inc_map; /* ICA_MAP */
			atags[10].ti_Data = RIGHTIMAGE;

			arrow_inc = NewObjectA(GadToolsBase->arrowclass, NULL, atags);
			if (!arrow_inc)
				goto failure;

			DEBUG_CREATESCROLLER(bug("scrollerg::New: arrow_inc gadget 0x%lx\n",arrow_inc));

		} /* if (scroller is FREEVERT or FREEHORIZ) */

	} /* if (scroller should have arrows attached) */

	newmsg.MethodID = OM_NEW;
	newmsg.ops_AttrList = sgtags;
	newmsg.ops_GInfo = NULL;

	o = (Object *)DoSuperMethodA(cl, o, (Msg)&newmsg);
	if (o)
	{
		struct SGData *data = INST_DATA(cl, o);
		struct opMember addm;

		DEBUG_CREATESCROLLER(bug("scrollerg::New: scrollerg %p\n", o));

		data->scroller = scroller;
		//data->arrow_inc = arrow_inc;
		//data->arrow_dec = arrow_dec;
		data->immediate = immediate;
		data->relverify = relverify;

		addm.MethodID = OM_ADDMEMBER;
		addm.opam_Object = scroller;
		DoMethodA(o, (Msg)&addm);

		if (arrow_inc)
		{
			addm.opam_Object = arrow_inc;
			DoMethodA(o, (Msg)&addm);

			addm.opam_Object = arrow_dec;
			DoMethodA(o, (Msg)&addm);
		}

		ReturnPtr ("scroller::New", IPTR, (IPTR)o);
	}

failure:
	DisposeObject(scroller);
	DisposeObject(arrow_dec);
	DisposeObject(arrow_inc);

	ReturnPtr ("scroller::New", IPTR, 0);
}

/**********************************************************************************************/

STATIC IPTR scrollerg_set(Class * cl, Object * o, struct opUpdate * msg)
{
	IPTR		retval = 0UL;
	struct TagItem	*tag, *tstate;
	struct TagItem	stags[] =
	    {
		    {TAG_IGNORE, 0},
		    {TAG_IGNORE,	0	},
		    {TAG_IGNORE,	0	},
		    {TAG_IGNORE,	0	},
		    {TAG_IGNORE,	0	},
		    {TAG_DONE		}
	    };
	BOOL has_stag = FALSE, has_non_stag = FALSE;

	D(bug("scrollerg_set(cl 0x%lx o 0x%lx msg 0x%lx)\n",cl,o,msg));

	tstate = msg->opu_AttrList;

	while ((tag = NextTagItem(&tstate)))
	{
		D(bug("scrollerg_set: tag 0x%08lx 0x%08lx\n", tag->ti_Tag, tag->ti_Data));

		switch (tag->ti_Tag)
		{
		case PGA_Top:
		case GTSC_Top:
			stags[0] = *tag;
			has_stag = TRUE;
			break;

		case PGA_Total:
		case GTSC_Total:
			stags[1] = *tag;
			has_stag = TRUE;
			break;

		case PGA_Visible:
		case GTSC_Visible:
			stags[2] = *tag;
			has_stag = TRUE;
			break;

		case ICA_TARGET:
			stags[3] = *tag;
			has_stag = TRUE;
			break;
		case ICA_MAP:
			stags[4] = *tag;
			has_stag = TRUE;
			break;

		default:
			has_non_stag = TRUE;
			break;
		}

	}

	if (has_stag)
	{
		struct opUpdate method;
		struct SGData *data = INST_DATA(cl, o);

		D(bug("scrollerg_set: set scroller tags\n"));

		method.MethodID = msg->MethodID;
		method.opu_AttrList = stags;
		method.opu_GInfo = msg->opu_GInfo;
		method.opu_Flags = msg->opu_Flags; /* only valid for OM_UPDATE. Oh well. */

		retval = DoMethodA(data->scroller, (Msg)&method);
	}

	if (has_non_stag)
	{
		D(bug("scrollerg_set: set superclass tags\n"));

		retval += DoSuperMethodA(cl, o, (Msg)msg);
	}

	return retval;
}

/**********************************************************************************************/

STATIC IPTR scrollerg_get(Class * cl, Object * o, struct opGet *msg)
{
	IPTR		retval;

	switch(msg->opg_AttrID)
	{
	case GTA_GadgetKind:
	case GTA_ChildGadgetKind:
		*msg->opg_Storage = SCROLLER_KIND;
		retval = 1UL;
		break;

	case GTSC_Top:
	case GTSC_Total:
	case GTSC_Visible:
		{
			struct SGData *data = INST_DATA(cl, o);
			retval = DoMethodA(data->scroller, (Msg)msg);
		}
		break;

	default:
		retval = DoSuperMethodA(cl, o, (Msg)msg);
		break;
	}

	return retval;
}

/**********************************************************************************************/

STATIC IPTR scrollerg_gadgetidcmpmsg(Class *cl, Object *o, struct gtpGadgetIDCMPMsg *msg)
{
	struct SGData *data = INST_DATA(cl, o);
	IPTR val;

	switch (msg->IntuiMsg->Class)
	{
	case IDCMP_GADGETDOWN:
		GetAttr(GTSC_Top, data->scroller, &val);
		if (!data->immediate)
		{
			if (data->code == (UWORD)val)
				return TRUE;
			msg->IntuiMsg->Class = IDCMP_MOUSEMOVE;
		}
		msg->IntuiMsg->Code = data->code = (UWORD)val;
		break;

	case IDCMP_MOUSEMOVE:
	case IDCMP_INTUITICKS:
		GetAttr(GTSC_Top, data->scroller, &val);
		if (data->code == (UWORD)val)
			return TRUE;
		msg->IntuiMsg->Class = IDCMP_MOUSEMOVE;
		msg->IntuiMsg->Code = data->code = (UWORD)val;
		break;

	case IDCMP_GADGETUP:
		GetAttr(GTSC_Top, data->scroller, &val);
		if (!data->relverify)
		{
			if (data->code == (UWORD)val)
				return TRUE;
			msg->IntuiMsg->Class = IDCMP_MOUSEMOVE;
		}
		msg->IntuiMsg->Code = data->code = (UWORD)val;
		break;
	}

	return FALSE;
}

/**********************************************************************************************/

AROS_UFH3S(IPTR, dispatch_scrollergclass,
		   AROS_UFHA(Class *,  cl,  A0),
		   AROS_UFHA(Object *, o,   A2),
		   AROS_UFHA(Msg,      msg, A1)
		  )
{
	AROS_USERFUNC_INIT

	IPTR retval;

	D(bug("dispatch_scrollergclass: cl 0x%lx o 0x%lx msg 0x%lx\n",cl,o,msg));

	switch(msg->MethodID)
	{
	case OM_NEW:
		D(bug("dispatch_scrollergclass: OM_NEW\n"));
		retval = scrollerg_new(cl, o, (struct opSet *)msg);
		break;

	case OM_UPDATE:
	case OM_SET:
		D(bug("dispatch_scrollergclass: %s\n",
		      msg->MethodID == OM_SET ? "OM_SET" : "OM_UPDATE"));
		retval = (IPTR)scrollerg_set(cl, o, (struct opUpdate *)msg);
		break;

	case OM_GET:
		D(bug("dispatch_scrollergclass: OM_GET\n"));
		retval = (IPTR)scrollerg_get(cl, o, (struct opGet *)msg);
		break;

	case GTM_GadgetIDCMPMsg:
		D(bug("dispatch_scrollergclass: GTM_GadgetIDCMPMsg\n"));
		retval = scrollerg_gadgetidcmpmsg(cl, o, (struct gtpGadgetIDCMPMsg *)msg);
		break;

	default:
		retval = DoSuperMethodA(cl, o, msg);
		break;
	} /* switch */

	D(bug("dispatch_scrollergclass: retval 0x%lx\n",retval));

	return (retval);

	AROS_USERFUNC_EXIT
}  /* dispatch_Gtlvclass */

/**********************************************************************************************/

#undef GadToolsBase

Class *makescrollergclass(struct GadToolsBase_intern * GadToolsBase)
{
	Class *cl;

	ObtainSemaphore(&GadToolsBase->classsema);

	cl = GadToolsBase->scrollergclass;
	if (!cl)
	{
		if (makegroupclass(GadToolsBase) && makescrollerclass(GadToolsBase) &&
		        makearrowclass(GadToolsBase))
		{
			cl = MakeClass(NULL, NULL, GadToolsBase->groupclass, sizeof(struct SGData), 0UL);
			if (cl)
			{
				cl->cl_Dispatcher.h_Entry = (APTR) AROS_ASMSYMNAME(dispatch_scrollergclass);
				cl->cl_Dispatcher.h_SubEntry = NULL;
				cl->cl_UserData = (IPTR) GadToolsBase;

				GadToolsBase->scrollergclass = cl;
			}
		}
	}

	ReleaseSemaphore(&GadToolsBase->classsema);

	return cl;
}

/**********************************************************************************************/
